@@ -81,7 +81,7 @@
       pole = v.length;
       v.push([0, -1, 0, 0, 0]); 
       v.push([0,  1, 0, 0, 1]);
-      result.values = new Float32Array(this.flatten(v));
+      result.values = new Float32Array(rglwidgetClass.flatten(v));
       result.vertexCount = v.length;
       
       mod1 = segments*(sections - 1);
@@ -102,7 +102,7 @@
                  ((j + 1)*(sections - 1) - 1) % mod1,
                  ((j + 1)*(sections - 1) + sections - 2) % mod1]);
       }
-      result.it = new Uint16Array(this.flatten(it));
+      result.it = new Uint16Array(rglwidgetClass.flatten(it));
       
       for (i = 0; i < it.length; i++) {
         centers.push([0,0,0]);
@@ -140,13 +140,10 @@
                 [0, 4, 6, 2], 
                 [0, 1, 5, 4], 
                 [4, 5, 7, 6]], 
-          centers = [], i, j, k, result = {};
+          centers = [], i, j, k, 
+          i0, i1, i2,
+          normal, result = {};
        
-      result.values = new Float32Array(this.flatten(v));
-      result.vertexCount = v.length;
-      
-      result.ib = new Uint16Array(this.flatten(ib));
-      
       for (i = 0; i < ib.length; i++) {
         centers.push([0,0,0]);
         for (j = 0; j < 3; j++) { // x,y,z
@@ -155,20 +152,41 @@
           }
         }
       }
-      result.centers = centers;
+      result.centers = centers; 
+      result.values = new Float32Array(6*4*3*2);
+      result.vertexCount = 24;
+      result.vertices = new Array(24);
+      result.normals = new Array(24);
+      for (i=0; i < 6; i++) {
+        for (j=0; j < 4; j++) {
+          i0 = ib[i][j];
+          result.vertices[4*i + j] = v[i0];
+          i1 = ib[i][(j + 1) % 4];
+          i2 = ib[i][(j + 2) % 4];
+          if (j === 0)
+            normal = rglwidgetClass.normalize(rglwidgetClass.xprod(rglwidgetClass.vdiff(v[i1], v[i0]),
+                                  rglwidgetClass.vdiff(v[i2], v[i0])));
+          result.normals[4*i + j] = normal;
+          for (k=0; k < 3; k++) {
+            result.values[i*24 + j*6 + k] = v[i0][k];
+            result.values[i*24 + j*6 + 3 + k] = normal[k];
+          }
+        }
+        for (j=0; j<4; j++)
+          ib[i][j] = 4*i + j;
+      }
+      result.ib = new Uint16Array(rglwidgetClass.flatten(ib));
       
-      result.vOffsets = {vofs:0, cofs:-1, nofs:0, radofs:-1, oofs:-1,
-                         tofs:-1, nextofs:-1, pointofs:-1, stride:3};
+      result.vOffsets = {vofs:0, cofs:-1, nofs:3, radofs:-1, oofs:-1,
+                         tofs:-1, nextofs:-1, pointofs:-1, stride:6};
 
       result.f = [];
       result.indices = {};
 
       result.colorCount = 1;
-      result.type = "cube";
-      result.vertices = v;
+      result.type = "quads";
       this.cube = result;
       this.initShapeGL(this.cube);
-      
     };
     
 
@@ -188,51 +206,64 @@
     /* Initialize common sphere object from spheres object
     */
     rglwidgetClass.prototype.initShapeFromObj = function(shape, obj) {
-      var i, pass, f, mode;
-       shape.ofsLoc = obj.ofsLoc;
-       shape.texLoc = obj.texLoc;
-       shape.sampler = obj.sampler;
-       shape.uFogMode = obj.uFogMode;
-       shape.uFogColor = obj.uFogColor;
-       shape.uFogParms = obj.uFogParms;
-       shape.userAttribLocations = obj.userAttribLocations;
-       shape.userUniformLocations = obj.userUniformLocations;
-       shape.normLoc = obj.normLoc;
-       shape.clipLoc = obj.clipLoc;
-       shape.nextLoc = obj.nextLoc;
-       shape.pointLoc = obj.pointLoc;
-       shape.aspectLoc = obj.aspectLoc;
-       shape.lwdLoc = obj.lwdLoc;
-       shape.prog = obj.prog;
-       shape.material = obj.material;
-       shape.flags = obj.flags;
-       shape.someHidden = obj.someHidden;
-       shape.fastTransparency = obj.fastTransparency;
-       shape.nlights = obj.nlights;
-       shape.emission = obj.emission;
-       shape.emissionLoc = obj.emissionLoc;
-       shape.shininess = obj.shininess;
-       shape.shininessLoc = obj.shininessLoc;
-       shape.ambient = obj.ambient;
-       shape.ambientLoc = obj.ambientLoc;
-       shape.specular = obj.specular;
-       shape.specularLoc = obj.specularLoc;
-       shape.diffuse = obj.diffuse;
-       shape.diffuseLoc = obj.diffuseLoc;
-       shape.lightDir = obj.lightDir;
-       shape.lightDirLoc = obj.lightDirLoc;
-       shape.viewpoint = obj.viewpoint;
-       shape.viewpointLoc = obj.viewpointLoc;
-       shape.finite = obj.finite;
-       shape.finiteLoc = obj.finiteLoc;
-       shape.prMatLoc = obj.prMatLoc;
-       shape.mvMatLoc = obj.mvMatLoc;
-       shape.normMatLoc = obj.normMatLoc;
-       shape.frontLoc = obj.frontLoc;
-       shape.index_uint = false;
-       shape.is_transparent = obj.is_transparent;
-       shape.ignoreExtent = obj.ignoreExtent;
-      if ( shape.passes !== obj.passes ||
+      var i, pass, f, mode, self = this,
+        /* This function selects things that would be
+           the back, ignoring perspective -- this is what 
+           we want for the bounding box decoration. */
+        is_back = function(i) {
+                var normal = [].concat(shape.normals[i]),
+                  pt = shape.vertices[i];
+                normal.push(-rglwidgetClass.dotprod(normal, pt));
+                normal = rglwidgetClass.multVM(normal, self.normMatrix);
+                return normal[2] < 0 || (normal[2] === 0 && normal[0] < 0);
+              }; 
+      shape.ofsLoc = obj.ofsLoc;
+      shape.texLoc = obj.texLoc;
+      shape.texture = obj.texture;
+      shape.sampler = obj.sampler;
+      shape.uFogMode = obj.uFogMode;
+      shape.uFogColor = obj.uFogColor;
+      shape.uFogParms = obj.uFogParms;
+      shape.userAttribLocations = obj.userAttribLocations;
+      shape.userUniformLocations = obj.userUniformLocations;
+      shape.normLoc = obj.normLoc;
+      shape.invPrMatLoc = obj.invPrMatLoc;
+      shape.clipLoc = obj.clipLoc;
+      shape.nextLoc = obj.nextLoc;
+      shape.pointLoc = obj.pointLoc;
+      shape.aspectLoc = obj.aspectLoc;
+      shape.lwdLoc = obj.lwdLoc;
+      shape.prog = obj.prog;
+      shape.material = obj.material;
+      shape.flags = obj.flags;
+      shape.defFlags = obj.defFlags;
+      shape.someHidden = obj.someHidden;
+      shape.fastTransparency = obj.fastTransparency;
+      shape.nlights = obj.nlights;
+      shape.emission = obj.emission;
+      shape.emissionLoc = obj.emissionLoc;
+      shape.shininess = obj.shininess;
+      shape.shininessLoc = obj.shininessLoc;
+      shape.ambient = obj.ambient;
+      shape.ambientLoc = obj.ambientLoc;
+      shape.specular = obj.specular;
+      shape.specularLoc = obj.specularLoc;
+      shape.diffuse = obj.diffuse;
+      shape.diffuseLoc = obj.diffuseLoc;
+      shape.lightDir = obj.lightDir;
+      shape.lightDirLoc = obj.lightDirLoc;
+      shape.viewpoint = obj.viewpoint;
+      shape.viewpointLoc = obj.viewpointLoc;
+      shape.finite = obj.finite;
+      shape.finiteLoc = obj.finiteLoc;
+      shape.prMatLoc = obj.prMatLoc;
+      shape.mvMatLoc = obj.mvMatLoc;
+      shape.normMatLoc = obj.normMatLoc;
+      shape.frontLoc = obj.frontLoc;
+      shape.index_uint = false;
+      shape.is_transparent = obj.is_transparent;
+      shape.ignoreExtent = obj.ignoreExtent;
+      if (shape.passes !== obj.passes ||
           JSON.stringify( shape.pmode) !== JSON.stringify(obj.pmode)) {
         shape.passes = obj.passes;
         shape.pmode = obj.pmode;
@@ -273,14 +304,34 @@
               }
       	      break;
       	    case "filled":
-      	      f =  shape.it || shape.ib;
+      	      if (typeof shape.it !== "undefined")
+      	        f =  shape.it;
+      	      else if (typeof shape.ib !== "undefined") {
+      	        f.length = 1.5*shape.ib.length;
+                for (i=0; i < shape.ib.length/4; i++) {
+                  f[6*i] = shape.ib[4*i];
+                  f[6*i+1] = shape.ib[4*i + 1];
+                  f[6*i+2] = shape.ib[4*i + 2];
+                  f[6*i+3] = shape.ib[4*i];
+                  f[6*i+4] = shape.ib[4*i + 2];
+                  f[6*i+5] = shape.ib[4*i + 3];
+                }      	        
+      	      }
+      	      break;
       	    }              
-             shape.indices[mode] = new Uint16Array(f);
+            shape.indices[mode] = new Uint16Array(f);
           }
-           shape.f[pass] =  shape.indices[mode];
+        }
+      }       
+      for (pass = 0; pass < obj.passes; pass++) {
+        mode =  shape.pmode[pass];
+        shape.f[pass] =  shape.indices[mode];
+        if (typeof obj.draw_front !== "undefined" &&
+            !obj.draw_front) {
+          shape.f[pass] = shape.f[pass].filter(is_back);   
         }
       }
-      // console.log("Names in  shapes not in  shape:"+JSON.stringify(this.keydiff(obj,  shape)));
+      // console.log("Names in  shapes not in  shape:"+JSON.stringify(rglwidgetClass.keydiff(obj,  shape)));
        shape.initialized = true;
     };
 
@@ -306,6 +357,8 @@
       sub.opaque = [];
       sub.lights = [];
       sub.needsBegin = true;
+      if (typeof sub.objects !== "undefined")
+        sub.objects = [].concat(sub.objects); /* make sure it's an array */
       for (i=0; i < sub.objects.length; i++) {
         obj = this.getObj(sub.objects[i]);
         if (typeof obj === "undefined") {
@@ -323,36 +376,61 @@
         this.initCube();
       obj.cube = {id: obj.id + 0.1,
                     type: "quads",
-                    flags: this.f_has_fog,
+                    flags: obj.flags,
                     material: obj.material,
                     colors: [obj.colors[0]],
                     vertices: this.cube.vertices,
+                    normals: this.cube.normals,
+                    draw_front: obj.draw_front,
                     initialized: false
         };
+      if (this.getMaterial(obj.cube, "front") !==
+          this.getMaterial(obj.cube, "back"))
+        /* jshint bitwise: false */  
+        obj.cube.flags |= rglwidgetClass.f_is_twosided;
+        /* jshint bitwise: true */
+      this.scene.objects[obj.cube.id] = obj.cube;
       obj.ticks = {id: obj.id + 0.2,
                      type: "lines",
-                     flags: this.f_has_fog,
+                     flags: rglwidgetClass.f_has_fog,
                      material: obj.material,
-                     colors: (obj.colors.length > 1 ? obj.colors[1] : [obj.colors[0]]),
+                     colors: (obj.colors.length > 1 ? [obj.colors[1]] : [obj.colors[0]]),
                      axes: obj.axes,
                      initialized: false
       };
+      this.scene.objects[obj.ticks.id] = obj.ticks;
       obj.labels = {id: obj.id + 0.3,
                      type: "text",
-                     flags: this.f_has_fog + 
-                            this.f_fixed_size + 
-                            this.f_fixed_quads,
+                     flags: rglwidgetClass.f_has_fog + 
+                            rglwidgetClass.f_fixed_size + 
+                            rglwidgetClass.f_fixed_quads,
                      material: {lit: false},
-                     colors: (obj.colors.length > 1 ? obj.colors[1] : [obj.colors[0]]),
+                     colors: (obj.colors.length > 1 ? [obj.colors[1]] : [obj.colors[0]]),
                      cex: [[1]],
                      family: [["sans"]],
                      font: [[1]],
-                     adj: [[0.5, 0.5]],
+                     adj: [[0.5, 0.5, 0.5]],
                      ignoreExtent: true,
                      initialized: false
       };
+      this.scene.objects[obj.labels.id] = obj.labels;
       obj.initialized = true;
     };
+    
+    rglwidgetClass.prototype.initBackground = function(obj) {
+      var material, fl = obj.defFlags;
+      if (typeof obj.ids !== "undefined")
+        obj.quad = rglwidgetClass.flatten([].concat(obj.ids));
+      else if (obj.sphere) {
+        fl.has_normals = true;
+        fl.needs_vnormal = true;
+        obj.defFlags = fl;
+        material = obj.material;
+        material.front = "culled";
+        obj.vertices = [[0,0,0]];
+        obj.texcoords = [[0,0]];
+      }  
+    };
 
     /**
      * Initialize object for display
@@ -364,44 +442,45 @@
       }
       return this.initObj(this.getObj(id));
     };
-    
+
     /**
      * Initialize object for display
      * @param { Object } obj - object to initialize
      */
     rglwidgetClass.prototype.initObj = function(obj) {
-      var flags = obj.flags,
-          type = obj.type,
-          is_lit = this.isSet(flags, this.f_is_lit),
-          fat_lines = this.isSet(flags, this.f_fat_lines),
-          has_texture = this.isSet(flags, this.f_has_texture),
-          fixed_quads = this.isSet(flags, this.f_fixed_quads),
-          is_transparent = this.isSet(flags, this.f_is_transparent),
-          depth_sort = this.isSet(flags, this.f_depth_sort),
-          sprites_3d = this.isSet(flags, this.f_sprites_3d),
-          fixed_size = this.isSet(flags, this.f_fixed_size),
-          is_twosided = this.isSet(flags, this.f_is_twosided),
-          is_brush = this.isSet(flags, this.f_is_brush),
-          has_fog = this.isSet(flags, this.f_has_fog),
-          has_normals = typeof obj.normals !== "undefined", 
+      var type = obj.type, 
+          flags = obj.flags,
+          normals = obj.normals,
+          round_points = (typeof obj.material === "undefined") ?
+            false : this.getMaterial(obj, "point_antialias"),
+          has_indices = typeof obj.indices !== "undefined",
+          has_spheres = type === "spheres" || 
+                        (type === "background" && obj.sphere),
+          sprites_3d = rglwidgetClass.isSet(flags, rglwidgetClass.f_sprites_3d),
+          depth_sort = rglwidgetClass.isSet(flags, rglwidgetClass.f_depth_sort),
           gl = this.gl || this.initGL(),
-          polygon_offset,
+          fl, polygon_offset,
           texinfo, drawtype, nclipplanes, f, nrows, oldrows,
           i,j,v,v1,v2, mat, uri, matobj, pass, pmode,
-          dim, nx, nz, nrow;
+          dim, nx, nz, nrow, shaders;
 
     obj.initialized = true;
-
+    
     obj.someHidden = false; // used in selection
-    obj.is_transparent = is_transparent;
+    
+    this.expandBufferedFields(obj);
     
     if (type === "subscene")
       return;
       
+    obj.defFlags = fl = rglwidgetClass.getDefFlags(flags, type, normals, round_points);
+  
+    obj.is_transparent = fl.is_transparent;
+  
     if (type === "bboxdeco")
       return this.initBBox(obj);
       
-    if (type === "spheres" && typeof this.sphere === "undefined")
+    if (has_spheres && typeof this.sphere === "undefined")
       this.initSphere(16, 16);
 
     if (type === "light") {
@@ -413,35 +492,41 @@
     }
 
     if (type === "clipplanes") {
-      obj.vClipplane = this.flatten(this.cbind(obj.normals, obj.offsets));
+      obj.vClipplane = rglwidgetClass.flatten(rglwidgetClass.cbind(obj.normals, obj.offsets));
       return;
     }
 
-    if (type === "background" && typeof obj.ids !== "undefined") {
-      obj.quad = this.flatten([].concat(obj.ids));
-      return;
+    if (type === "background") {
+      this.initBackground(obj);
+      if (!obj.sphere)
+        return;
     }
 
     polygon_offset = this.getMaterial(obj, "polygon_offset");
     if (polygon_offset[0] !== 0 || polygon_offset[1] !== 0)
       obj.polygon_offset = polygon_offset;
 
-    if (is_transparent) {
+    if (fl.is_transparent) {
       depth_sort = ["triangles", "quads", "surface",
-                    "spheres", "sprites", "text"].indexOf(type) >= 0;
+                    "spheres", "sprites", "text",
+                    "planes"].indexOf(type) >= 0;
     }
     
-    if (is_brush)
+    if (fl.is_brush)
       this.initSelection(obj.id);
 
     if (typeof obj.vertices === "undefined")
       obj.vertices = [];
 
     v = obj.vertices;
-    obj.vertexCount = v.length;
+    if (has_indices)
+      obj.vertexCount = obj.indices.length;
+    else
+      obj.vertexCount = v.length;
+      
     if (!obj.vertexCount) return;
 
-    if (is_twosided && !has_normals) {
+    if (fl.is_twosided && !fl.has_normals && type !== "background") {
       if (typeof obj.userAttributes === "undefined")
         obj.userAttributes = {};
       v1 = Array(v.length);
@@ -485,11 +570,14 @@
         gl.deleteProgram(obj.prog);
         obj.prog = undefined;
       }
+      
+      shaders = this.getShaders(obj);
+      
       obj.prog = gl.createProgram();
       gl.attachShader(obj.prog, this.getShader( gl.VERTEX_SHADER,
-        this.getVertexShader(obj) ));
+                      shaders.vertex ));
       gl.attachShader(obj.prog, this.getShader( gl.FRAGMENT_SHADER,
-                      this.getFragmentShader(obj) ));
+                      shaders.fragment ));
       //  Force aPos to location 0, aCol to location 1
       gl.bindAttribLocation(obj.prog, 0, "aPos");
       gl.bindAttribLocation(obj.prog, 1, "aCol");
@@ -508,29 +596,34 @@
 
     if (type === "text") {
       texinfo = this.drawTextToCanvas(obj.texts,
-                                      this.flatten(obj.cex),
-                                      this.flatten(obj.family),
-                                      this.flatten(obj.family));
+                                      rglwidgetClass.flatten(obj.cex),
+                                      rglwidgetClass.flatten(obj.family),
+                                      rglwidgetClass.flatten(obj.family));
     }
 
-    if (fixed_quads && !sprites_3d) {
+    if (fl.fixed_quads && !sprites_3d) {
       obj.ofsLoc = gl.getAttribLocation(obj.prog, "aOfs");
     }
 
-    if (has_texture || type === "text") {
-      if (!obj.texture)
+    if (fl.has_texture || type === "text") {
+      if (!obj.texture) {
         obj.texture = gl.createTexture();
+        // This is a trick from https://stackoverflow.com/a/19748905/2554330 to avoid warnings
+        gl.bindTexture(gl.TEXTURE_2D, obj.texture);
+        gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, 1, 1, 0, gl.RGBA, gl.UNSIGNED_BYTE,
+              new Uint8Array([255,255,255, 255])); // white
+      }
       obj.texLoc = gl.getAttribLocation(obj.prog, "aTexcoord");
       obj.sampler = gl.getUniformLocation(obj.prog, "uSampler");
     }
     
-    if (has_fog && !sprites_3d) {
+    if (fl.has_fog && !sprites_3d) {
       obj.uFogMode = gl.getUniformLocation(obj.prog, "uFogMode");
       obj.uFogColor = gl.getUniformLocation(obj.prog, "uFogColor");
       obj.uFogParms = gl.getUniformLocation(obj.prog, "uFogParms");
     }
 
-    if (has_texture) {
+    if (fl.has_texture) {
       mat = obj.material;
       if (typeof mat.uri !== "undefined")
         uri = mat.uri;
@@ -553,7 +646,7 @@
 
     var stride = 3, nc, cofs, nofs, radofs, oofs, tofs, vnew, fnew,
         nextofs = -1, pointofs = -1, alias, colors, key, selection,
-        filter, adj, pos, offset, attr, last, options;
+        filter, adj, offset, attr, last, options;
 
     obj.alias = undefined;
     
@@ -589,16 +682,16 @@
     if (nc > 1) {
       cofs = stride;
       stride = stride + 4;
-      v = this.cbind(v, colors);
+      v = rglwidgetClass.cbind(v, colors);
     } else {
       cofs = -1;
-      obj.onecolor = this.flatten(colors);
+      obj.onecolor = rglwidgetClass.flatten(colors);
     }
 
-    if (has_normals) {
+    if (fl.has_normals && !has_spheres) {
       nofs = stride;
       stride = stride + 3;
-      v = this.cbind(v, typeof obj.pnormals !== "undefined" ? obj.pnormals : obj.normals);
+      v = rglwidgetClass.cbind(v, typeof obj.pnormals !== "undefined" ? obj.pnormals : obj.normals);
     } else
       nofs = -1;
 
@@ -607,7 +700,7 @@
       stride = stride + 1;
       // FIXME:  always concat the radii?
       if (obj.radii.length === v.length) {
-        v = this.cbind(v, obj.radii);
+        v = rglwidgetClass.cbind(v, obj.radii);
       } else if (obj.radii.length === 1) {
         v = v.map(function(row) { return row.concat(obj.radii[0]);});
       }
@@ -615,33 +708,57 @@
       radofs = -1;
       
     // Add default indices
-    f = Array(v.length);
-    for (i = 0; i < v.length; i++)
-      f[i] = i;
+    if (has_indices) {
+      f = Array(obj.indices.length);
+      for (i = 0; i < f.length; i++)
+        f[i] = obj.indices[i] - 1;
+    } else {
+      f = Array(v.length);
+      for (i = 0; i < v.length; i++)
+        f[i] = i;
+    }
     obj.f = [f,f];
 
     if (type === "sprites" && !sprites_3d) {
       tofs = stride;
       stride += 2;
       oofs = stride;
-      stride += 2;
+      stride += 3;
       vnew = new Array(4*v.length);
       fnew = new Array(4*v.length);
       alias = new Array(v.length);
-      var rescale = fixed_size ? 72 : 1,
+      var rescale = fl.fixed_size ? 72 : 1,
           size = obj.radii, s = rescale*size[0]/2;
       last = v.length;
       f = obj.f[0];
+      obj.adj = rglwidgetClass.flatten(obj.adj);
+      if (typeof obj.pos !== "undefined") {
+        obj.pos = rglwidgetClass.flatten(obj.pos);
+        offset = obj.adj[0];
+      } else
+        offset = 0;
       for (i=0; i < v.length; i++) {
+        adj = this.getAdj(obj, i, offset);
         if (size.length > 1)
           s = rescale*size[i]/2;
-        vnew[i]  = v[i].concat([0,0,-s,-s]);
+        adj[0] = 2*s*(adj[0] - 0.5);
+        adj[1] = 2*s*(adj[1] - 0.5);
+        adj[2] = 2*s*(adj[2] - 0.5);
+        vnew[i]  = v[i].concat([0,0]).concat([-s-adj[0],
+                                              -s-adj[1],
+                                              -adj[2]]);
         fnew[4*i] = f[i];
-        vnew[last]= v[i].concat([1,0, s,-s]);
+        vnew[last]= v[i].concat([1,0]).concat([s-adj[0],
+                                              -s-adj[1],
+                                              -adj[2]]);
         fnew[4*i+1] = last++;
-        vnew[last]= v[i].concat([1,1, s, s]);
+        vnew[last]= v[i].concat([1,1]).concat([s-adj[0],
+                                               s-adj[1],
+                                               -adj[2]]);
         fnew[4*i+2] = last++;
-        vnew[last]= v[i].concat([0,1,-s, s]);
+        vnew[last]= v[i].concat([0,1]).concat([-s-adj[0],
+                                                s-adj[1],
+                                                -adj[2]]);
         fnew[4*i+3] = last++;
         alias[i] = [last-3, last-2, last-1];
       }
@@ -652,20 +769,20 @@
       tofs = stride;
       stride += 2;
       oofs = stride;
-      stride += 2;
+      stride += 3;
       vnew = new Array(4*v.length);
       f = obj.f[0];
       fnew = new Array(4*f.length);
       alias = new Array(v.length);
       last = v.length;
-      adj = this.flatten(obj.adj);
+      adj = rglwidgetClass.flatten(obj.adj);
       if (typeof obj.pos !== "undefined") {
-        pos = this.flatten(obj.pos);
+        obj.pos = rglwidgetClass.flatten(obj.pos);
         offset = adj[0];
-      }
+      } else
+        offset = 0;
       for (i=0; i < v.length; i++) {
-        if (typeof pos !== "undefined")
-          adj = this.getAdj(pos[i % pos.length], offset, obj.texts[i]);
+        adj = this.getAdj(obj, i, offset, obj.texts[i]);
         vnew[i]  = v[i].concat([0,-0.5]).concat(adj);
         fnew[4*i] = f[i];
         vnew[last] = v[i].concat([1,-0.5]).concat(adj);
@@ -692,7 +809,7 @@
       tofs = stride;
       stride += 2;
       oofs = -1;
-      v = this.cbind(v, obj.texcoords);
+      v = rglwidgetClass.cbind(v, obj.texcoords);
     } else {
       tofs = -1;
       oofs = -1;
@@ -708,71 +825,85 @@
       	obj.userAttribLocations[attr] = gl.getAttribLocation(obj.prog, attr);
       	if (obj.userAttribLocations[attr] >= 0) { // Attribute may not have been used
       	  obj.userAttribOffsets[attr] = stride;
-      	  v = this.cbind(v, obj.userAttributes[attr]);
+      	  v = rglwidgetClass.cbind(v, obj.userAttributes[attr]);
       	  stride = v[0].length;
       	  obj.userAttribSizes[attr] = stride - obj.userAttribOffsets[attr];
-      	}
+      	} else
+      	  console.warn("attribute '"+attr+"' not found in object "+obj.id+".");
       }
     }
 
-    if (typeof obj.userUniforms !== "undefined") {
+    if (typeof obj.userUniforms !== "undefined" ||
+        typeof obj.userTextures !== "undefined") {
       obj.userUniformLocations = {};
-      for (attr in obj.userUniforms)
+      for (attr in obj.userUniforms) {
         obj.userUniformLocations[attr] = gl.getUniformLocation(obj.prog, attr);
+        if (obj.userUniformLocations[attr] === null)
+          console.warn("uniform '"+attr+"' not found in object "+obj.id+".");
+      }
+      for (attr in obj.userTextures) {
+        var texture = obj.userTextures[attr];
+        texture.texture = gl.createTexture();
+        // This is a trick from https://stackoverflow.com/a/19748905/2554330 to avoid warnings
+        gl.bindTexture(gl.TEXTURE_2D, texture.texture);
+        gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, 1, 1, 0, gl.RGBA, gl.UNSIGNED_BYTE,
+              new Uint8Array([255,255,255, 255])); // white
+        texture.sampler = gl.getUniformLocation(obj.prog, attr);
+        if (texture.sampler === null)
+          console.warn("sampler '"+attr+"' not found in object "+obj.id+".");
+        uri = texture.uri;
+        this.loadImageToTexture(uri, texture.texture);
+      }
     }
 
     if (sprites_3d) {
-      obj.userMatrix = new CanvasMatrix4(obj.usermatrix);
-      obj.objects = this.flatten([].concat(obj.ids));
-      is_lit = false;
+      obj.userMatrix = new CanvasMatrix4();
+      obj.userMatrix.load(rglwidgetClass.flatten(obj.usermatrix));
+      obj.objects = rglwidgetClass.flatten([].concat(obj.ids));
+      fl.is_lit = false;
+      obj.adj = rglwidgetClass.flatten(obj.adj);
+      if (typeof obj.pos !== "undefined") {
+        obj.pos = rglwidgetClass.flatten(obj.pos);
+        obj.offset = obj.adj[0];
+      } else
+        obj.offset = 0;
+        
       for (i=0; i < obj.objects.length; i++)
         this.initObjId(obj.objects[i]);
     }
 
-    if (is_lit && !fixed_quads) {
-       obj.normLoc = gl.getAttribLocation(obj.prog, "aNorm");
-    }
-
     nclipplanes = this.countClipplanes();
     if (nclipplanes && !sprites_3d) {
-      obj.clipLoc = [];
-      for (i=0; i < nclipplanes; i++)
-        obj.clipLoc[i] = gl.getUniformLocation(obj.prog,"vClipplane" + i);
+      obj.clipLoc = gl.getUniformLocation(obj.prog,"vClipplane");
     }
 
-    if (is_lit) {
+    if (fl.is_lit) {
       obj.emissionLoc = gl.getUniformLocation(obj.prog, "emission");
       obj.emission = new Float32Array(this.stringToRgb(this.getMaterial(obj, "emission")));
       obj.shininessLoc = gl.getUniformLocation(obj.prog, "shininess");
       obj.shininess = this.getMaterial(obj, "shininess");
       obj.nlights = this.countLights();
-      obj.ambientLoc = [];
-      obj.ambient = new Float32Array(this.stringToRgb(this.getMaterial(obj, "ambient")));
-      obj.specularLoc = [];
-      obj.specular = new Float32Array(this.stringToRgb(this.getMaterial(obj, "specular")));
-      obj.diffuseLoc = [];
-      obj.lightDirLoc = [];
-      obj.viewpointLoc = [];
-      obj.finiteLoc = [];
-      for (i=0; i < obj.nlights; i++) {
-        obj.ambientLoc[i] = gl.getUniformLocation(obj.prog, "ambient" + i);
-        obj.specularLoc[i] = gl.getUniformLocation(obj.prog, "specular" + i);
-        obj.diffuseLoc[i] = gl.getUniformLocation(obj.prog, "diffuse" + i);
-        obj.lightDirLoc[i] = gl.getUniformLocation(obj.prog, "lightDir" + i);
-        obj.viewpointLoc[i] = gl.getUniformLocation(obj.prog, "viewpoint" + i);
-        obj.finiteLoc[i] = gl.getUniformLocation(obj.prog, "finite" + i);
+      if (obj.nlights > 0) {
+        obj.ambient = new Float32Array(this.stringToRgb(this.getMaterial(obj, "ambient")));
+        obj.specular = new Float32Array(this.stringToRgb(this.getMaterial(obj, "specular")));
+        obj.ambientLoc = gl.getUniformLocation(obj.prog, "ambient");
+        obj.specularLoc = gl.getUniformLocation(obj.prog, "specular");
+        obj.diffuseLoc = gl.getUniformLocation(obj.prog, "diffuse" );
+        obj.lightDirLoc = gl.getUniformLocation(obj.prog, "lightDir");
+        obj.viewpointLoc = gl.getUniformLocation(obj.prog, "viewpoint");
+        obj.finiteLoc = gl.getUniformLocation(obj.prog, "finite" );
       }
     }
     
-    obj.passes = is_twosided + 1;
+    obj.passes = fl.is_twosided + 1;
     obj.pmode = new Array(obj.passes);
     for (pass = 0; pass < obj.passes; pass++) {
-      if (type === "triangles" || type === "quads" || type === "surface" || type === "spheres")
+      if (type === "triangles" || type === "quads" || type === "surface" || has_spheres)
       	pmode = this.getMaterial(obj, (pass === 0) ? "front" : "back");
       else pmode = "filled";
       obj.pmode[pass] = pmode;
     }
-    if (type !== "spheres") {
+    if (!has_spheres) {
       obj.f.length = obj.passes;
       for (pass = 0; pass < obj.passes; pass++) {
       	f = fnew = obj.f[pass];
@@ -825,7 +956,7 @@
       	      fnew[6*i + 5] = f[3*i];
       	    }
           }
-        } else if (type === "spheres") {
+        } else if (has_spheres) {
           // default
         } else if (type === "surface") {
           dim = obj.dim[0];
@@ -866,7 +997,7 @@
       }
     }
     
-    if (fat_lines) {
+    if (fl.fat_lines) {
       alias = undefined;
       obj.nextLoc = gl.getAttribLocation(obj.prog, "aNext");
       obj.pointLoc = gl.getAttribLocation(obj.prog, "aPoint");
@@ -985,9 +1116,9 @@
     obj.vOffsets = {vofs:0, cofs:cofs, nofs:nofs, radofs:radofs, oofs:oofs, tofs:tofs,
                     nextofs:nextofs, pointofs:pointofs, stride:stride};
 
-    obj.values = new Float32Array(this.flatten(v));
+    obj.values = new Float32Array(rglwidgetClass.flatten(v));
 
-    if (type !== "spheres" && !sprites_3d) {
+    if (!has_spheres && !sprites_3d) {
       obj.buf = gl.createBuffer();
       gl.bindBuffer(gl.ARRAY_BUFFER, obj.buf);
       gl.bufferData(gl.ARRAY_BUFFER, obj.values, gl.STATIC_DRAW); //
@@ -995,7 +1126,7 @@
       obj.ibuf[0] = gl.createBuffer();
       gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, obj.ibuf[0]);
       gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, obj.f[0], gl[drawtype]);
-      if (is_twosided) {
+      if (fl.is_twosided) {
       	obj.ibuf[1] = gl.createBuffer();
       	gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, obj.ibuf[1]);
       	gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, obj.f[1], gl[drawtype]);
@@ -1005,18 +1136,21 @@
     if (!sprites_3d) {
       obj.mvMatLoc = gl.getUniformLocation(obj.prog, "mvMatrix");
       obj.prMatLoc = gl.getUniformLocation(obj.prog, "prMatrix");
+
+      if (fl.fixed_size) {
+        obj.textScaleLoc = gl.getUniformLocation(obj.prog, "textScale");
+      }
     }
 
-    if (fixed_size) {
-      obj.textScaleLoc = gl.getUniformLocation(obj.prog, "textScale");
-    }
-
-    if (is_lit && !sprites_3d) {
+    if (fl.needs_vnormal) {
+      obj.normLoc = gl.getAttribLocation(obj.prog, "aNorm");
       obj.normMatLoc = gl.getUniformLocation(obj.prog, "normMatrix");
     }
 
-    if (is_twosided) {
+    if (fl.is_twosided) {
       obj.frontLoc = gl.getUniformLocation(obj.prog, "front");
+      if (fl.has_normals)
+        obj.invPrMatLoc = gl.getUniformLocation(obj.prog, "invPrMatrix");
     }
   };
     
@@ -1030,6 +1164,7 @@
       this.textureCanvas.style.display = "block";
       this.scene = x;
       this.normMatrix = new CanvasMatrix4();
+      this.invPrMatrix = new CanvasMatrix4();
       this.saveMat = {};
       this.distance = null;
       this.posLoc = 0;
@@ -1119,6 +1254,14 @@
       this.restartCanvas();
       var objs = this.scene.objects,
           self = this;
+          
+      /* These hold context specific data.  In Shiny, they   
+         need to be deleted.  Elsewhere, they don't exist
+         and these are no-ops. */
+         
+      delete this.cube;
+      delete this.sphere;
+      
       Object.keys(objs).forEach(function(key){
         self.initSubscene(parseInt(key, 10));
       });
