@@ -36,6 +36,8 @@
 #ifndef __GL2PS_H__
 #define __GL2PS_H__
 
+#ifndef RGL_NO_OPENGL
+
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -255,4 +257,6 @@ GL2PSDLL_API GLint gl2psGetFileFormat();
 }
 #endif
 
+#endif 
+
 #endif /* __GL2PS_H__ */
